import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class ScrollDemo extends Canvas implements Runnable
{
  /* some constants */
  static final int WIDTH  = 450;     // width of component
  static final int HEIGHT = 300;     // height of component
  static final int ANIM_DELAY = 10;  // in milliseconds
  static final int ANIM_STEPS = 400;

  /* state of component */
  static final int STATE_UNINITIALIZED = 0;
  static final int STATE_LOADED        = 1;
  static final int STATE_READY         = 2;

  int state = STATE_UNINITIALIZED;

  Image          image;         // loaded image
  BufferedImage  cached_image;  // cached version of this

  SineTable table;  // table of sine value
  int anim_step;    // current position in table

  public ScrollDemo()
  {
    super();
    image = getToolkit().getImage("logo.jpg");

    // wait for image to load
    try {
      MediaTracker tracker = new MediaTracker(this);
      tracker.addImage(image, 0);
      tracker.waitForID(0);
    } catch (Exception e) { System.exit(0);}

    this.state = STATE_LOADED;

    // start the animation
    this.start();
  }

  /*
   * We'd like to be WIDTHxHEIGHT large, please
   */
  public Dimension getPreferredSize()
  {
    return new Dimension( WIDTH, HEIGHT );
  }

  /*
   * Draw ourselves
   */
  public void paint( Graphics g )
  {
    switch( state )
    {
        case STATE_UNINITIALIZED:
          // Please wait, loading images . . .
          break;
        case STATE_LOADED:
          /* render image into a compatabile image buffer
           * ready for display */
          cached_image = (BufferedImage) this.createImage( image.getWidth(this), image.getHeight(this) );
          cached_image.createGraphics().drawImage( image, 0, 0, this );

          /* generate sine table for the motion */
	  table = new SineTable( image.getWidth(this)-WIDTH, ANIM_STEPS );

	  state = STATE_READY;
	  /* fall through */
	case STATE_READY:
          /* create sub-image that is the size of our component */
          /* this is offset horizontally by an amount got from the sine table */
          BufferedImage sub_image = cached_image.getSubimage( table.get(anim_step),0, Math.min(WIDTH,image.getWidth(this)), Math.min(HEIGHT,image.getHeight(this)) );

          /* draw this sub-image to our component */
          g.drawImage( sub_image,0,0,this );
	  break;
	default:
    }
  }

  long last_update; // time of last call to update()

  public void update( Graphics g )
  {
    // just let paint() do the drawing
    paint(g);

    // dodgy frames per second hack
    long curr_time=System.currentTimeMillis();
    g.setColor( Color.black );
    g.drawString( Integer.toString((int)(1000/(curr_time-last_update))),10,10 );
    last_update=curr_time;
  }

  /*
   * do the animation
   */
  public void run()
  {
    long t = System.currentTimeMillis();

    /* loop forever (or until interrupted) */
    while( true )
    {
      switch( state )
      {
        case STATE_READY:
          // update offset in sine table
          anim_step=(anim_step+1)%table.length();

          /* send redraw message */
          repaint();
          break;

        default:
         break;
      }

      try
      {
        // delay at most ANIM_DELAY milliseconds
        t += ANIM_DELAY;
	Thread.sleep( Math.max( 0, t - System.currentTimeMillis() ) );
      }
      catch( InterruptedException e ) { break; }
    } /* while */
  }

  /*
   * start animation thread
   */
  public void start()
  {
    new Thread(this).start();
  }

  /*
   * Start here!
   */
  public static void main( String args[] )
  {
    Frame win = new Frame( "ScrollDemo" );
    win.addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(java.awt.event.WindowEvent e)
                             {
                               System.exit(0);
                             }
                           } );
    win.add( new ScrollDemo() );
    win.pack();
    win.show();
    win.setResizable( false );
  }
}
